@@ -1,14 +1,16 @@
 package Gimp::UI;
 
 use Carp;
-use Gimp;
+use Gimp ();
 use Gtk;
 
+$gtk_10 = Gtk->major_version==1 && Gtk->minor_version==0;
+
 $VERSION = $Gimp::VERSION;
 
 =head1 NAME
 
-Gimp::UI - "simulation of libgimpui"
+Gimp::UI - "simulation of libgimpui", and more!
 
 =head1 SYNOPSIS
 
@@ -21,15 +23,17 @@ reimplement all of it in perl.
 
 =over 4
 
- $option_menu = new Gimp::UI::ImageMenu;
- $option_menu = new Gimp::UI::LayerMenu;
- $option_menu = new Gimp::UI::ChannelMenu;
- $option_menu = new Gimp::UI::DrawableMenu;
+ $option_menu = new Gimp::UI::ImageMenu
+ $option_menu = new Gimp::UI::LayerMenu
+ $option_menu = new Gimp::UI::ChannelMenu
+ $option_menu = new Gimp::UI::DrawableMenu (constraint_func, active_element, \var);
  
  $button = new Gimp::UI::PatternSelect;
  $button = new Gimp::UI::BrushSelect;
  $button = new Gimp::UI::GradientSelect;
 
+ $button = new Gimp::UI::ColorSelectButton;
+
 =back
 
 =cut
@@ -39,44 +43,50 @@ reimplement all of it in perl.
 @Gimp::UI::ChannelMenu::ISA =qw(Gimp::UI);
 @Gimp::UI::DrawableMenu::ISA=qw(Gimp::UI);
 
+sub image_name {
+   my $name = $_[0]->get_filename;
+   $name.="-".${$_[0]} if $name eq "Untitled";
+   $name;
+}
+
 sub Gimp::UI::ImageMenu::_items {
-  map [[$_],$_,$_->get_filename],
+  map [[$_],$_,image_name($_)],
       Gimp->list_images ();
 }
 sub Gimp::UI::LayerMenu::_items {
-  map { my $i = $_; map [[$i,$_],$_,$i->get_filename."/".$_->get_name],$i->get_layers }
+  map { my $i = $_; map [[$i,$_],$_,image_name($i)."/".$_->get_name],$i->get_layers }
       Gimp->list_images ();
 }
 
 sub Gimp::UI::ChannelMenu::_items {
-  map { my $i = $_; map [[$i,$_],$_,$i->get_filename."/".$_->get_name],$i->get_channels }
+  map { my $i = $_; map [[$i,$_],$_,image_name($i)."/".$_->get_name],$i->get_channels }
       Gimp->list_images ();
 }
 
 sub Gimp::UI::DrawableMenu::_items {
-  map { my $i = $_; map [[$i,$_],$_,$i->get_filename."/".$_->get_name],($i->get_layers, $i->get_channels) }
+  map { my $i = $_; map [[$i,$_],$_,image_name($i)."/".$_->get_name],($i->get_layers, $i->get_channels) }
       Gimp->list_images ();
 }
 
 sub new($$$$) {
-   my($class,$constraint,$active)=@_;
+   my($class,$constraint,$active,$var)=@_;
    my(@items)=$class->_items;
    my $menu = new Gtk::Menu;
    for(@items) {
       my($constraints,$result,$name)=@$_;
       next unless $constraint->(@{$constraints});
       my $item = new Gtk::MenuItem $name;
-      $item->show;
-      $item->signal_connect(activate => sub { $_[3]=$result });
+      $item->signal_connect(activate => sub { $$var=$result });
       $menu->append($item);
    }
    if (@items) {
-      $_[3]=$items[0]->[1];
+      $$var=$items[0]->[1];
    } else {
       my $item = new Gtk::MenuItem "(none)";
-      $item->show;
       $menu->append($item);
+      $$var=undef;
    }
+   $menu->show_all;
    $menu;
 }
 
@@ -85,7 +95,17 @@ package Gimp::UI::PreviewSelect;
 use Gtk;
 use base 'Gtk::Button';
 
-register_type Gimp::UI::PreviewSelect;
+# this is an utter HACK for the braindamanged gtk (NOT Gtk!)
+sub register_types {
+   unless ($once) {
+      $once=1;
+      Gtk::Button->register_subtype(Gimp::UI::PreviewSelect);
+      Gtk::Button->register_subtype(Gimp::UI::PatternSelect);
+      Gtk::Button->register_subtype(Gimp::UI::BrushSelect);
+      Gtk::Button->register_subtype(Gimp::UI::GradientSelect);
+      Gtk::Button->register_subtype(Gimp::UI::ColorSelectButton);
+   }
+}
 
 sub GTK_CLASS_INIT {
    my $class = shift;
@@ -150,7 +170,7 @@ sub GTK_OBJECT_INIT {
       $l->selection and
          $self->set_preview($l->selection->children->get);
    });
-   $s->add ($l);
+   $gtk_10 ? $s->add ($l) : $s->add_with_viewport ($l);
    
    my $button = new Gtk::Button "OK";
    signal_connect $button "clicked", sub {
@@ -178,8 +198,6 @@ package Gimp::UI::PatternSelect;
 use Gtk;
 use base 'Gimp::UI::PreviewSelect';
 
-register_type Gimp::UI::PatternSelect;
-
 sub get_title { "Pattern Selection Dialog" }
 sub get_list { Gimp->patterns_list }
 
@@ -219,6 +237,7 @@ sub set_preview {
 }
 
 sub new {
+   Gimp::UI::PreviewSelect::register_types;
    new Gtk::Widget @_;
 }
 
@@ -227,8 +246,6 @@ package Gimp::UI::BrushSelect;
 use Gtk;
 use base 'Gimp::UI::PreviewSelect';
 
-register_type Gimp::UI::BrushSelect;
-
 sub get_title { "Brush Selection Dialog" }
 sub get_list { Gimp->brushes_list }
 
@@ -263,6 +280,7 @@ sub set_preview {
 }
 
 sub new {
+   Gimp::UI::PreviewSelect::register_types;
    new Gtk::Widget @_;
 }
 
@@ -271,8 +289,6 @@ package Gimp::UI::GradientSelect;
 use Gtk;
 use base 'Gimp::UI::PreviewSelect';
 
-register_type Gimp::UI::GradientSelect;
-
 sub get_title { "Gradient Selection Dialog" }
 sub get_list { keys %gradients }
 
@@ -288,15 +304,139 @@ sub set_preview {
 }
 
 sub new {
+   Gimp::UI::PreviewSelect::register_types;
    unless (defined %gradients) {
       undef @gradients{Gimp->gradients_get_list};
    }
    new Gtk::Widget @_;
 }
 
+package Gimp::UI::ColorSelectButton;
+
+use strict;
+use vars qw($VERSION @ISA);
+use Gtk;
+
+@ISA = qw(Gtk::Button);
+
+# Class defaults data
+my @class_def_color = (255,175,0);
+
+sub GTK_CLASS_INIT {
+	my($class) = shift;
+	
+	if (Gtk->major_version < 1 or (Gtk->major_version == 1 and Gtk->minor_version < 1)) {
+		add_arg_type $class "color", "string", 3; #R/W
+	} else {
+		add_arg_type $class "color", "GtkString", 3; #R/W
+	}
+}
+
+sub GTK_OBJECT_INIT {
+    my (@color) = @class_def_color;
+    
+    my($color_button) = @_;
+    
+    $color_button->{_color} ||= [@color];
+
+    my $preview = new Gtk::Preview -color;
+    
+    $color_button->{_preview} = $preview;
+    
+    # work around funny bug somewhere in gtk...
+    $preview->size(300,50);
+    $preview->show;
+    $color_button->add($preview);
+        
+    signal_connect $color_button "size_allocate" => sub {
+    	my($self,$allocation) = @_;
+    	my($x,$y,$w,$h) = @$allocation;
+    	$w -= 6;
+    	$h -= 6;
+    	$self->{_preview_width} = $w;
+    	$self->{_preview_height} = $h;
+        $self->{_preview}->size($self->{_preview_width},$self->{_preview_height});
+    	$self->update_color;
+    };
+    
+    signal_connect $color_button "clicked" => \&cb_color_button;
+}
+
+sub GTK_OBJECT_SET_ARG {
+   my($self,$arg,$id, $value) = @_;
+   $self->{_color} = [split(' ',$value)];
+   $self->update_color;
+}
+
+sub GTK_OBJECT_GET_ARG {
+   my($self,$arg,$id) = @_;
+   return join(' ',@{$self->{_color}});
+}
+
+sub update_color($) {
+    my($this) = shift;
+    
+    return unless defined $this->{_preview} and defined $this->{_preview_width};
+    
+    my($preview, $color) = ($this->{_preview}, $this->{_color});
+    my($width, $height) = ($this->{_preview_width}, $this->{_preview_height});
+    
+    my($buf) = pack("C3", @$color) x $width;
+
+    for(0..$height-1) {
+       $preview->draw_row($buf, 0, $_, $width);
+    }
+    $preview->draw(undef);
+}
+
+sub color_selection_ok {
+    my($widget, $dialog, $color_button) = @_;
+	
+    my(@color) = $dialog->colorsel->get_color;
+    @{$color_button->{_color}} = map(int(255.99*$_),@color);
+
+    $color_button->update_color();
+    $dialog->destroy();
+    delete $color_button->{_cs_window};
+}
+
+sub cb_color_button {
+    my($color_button) = @_;
+    
+    if (defined $color_button->{_cs_window}) {
+    	if (!$color_button->{_cs_window}->mapped) {
+	    	$color_button->{_cs_window}->hide;
+	    }
+    	$color_button->{_cs_window}->show;
+    	$color_button->{_cs_window}->window->raise;
+    	return;
+    }
+
+    my $cs_window=new Gtk::ColorSelectionDialog("Color");
+    $cs_window->colorsel->set_color(map($_*1/255,@{$color_button->{_color}}));
+    $cs_window->show();
+    $cs_window->ok_button->signal_connect("clicked",
+					  \&color_selection_ok,
+					  $cs_window,
+					  $color_button);
+    $cs_window->cancel_button->signal_connect("clicked",
+					      sub { $cs_window->destroy; delete $color_button->{_cs_window} });
+    $color_button->{_cs_window} = $cs_window;
+}
+
+sub new {
+    my $pkg = shift;
+   Gimp::UI::PreviewSelect::register_types;
+    return new Gtk::Widget $pkg, @_;
+}
+
+1;
+
 =head1 AUTHOR
 
-Marc Lehmann <pcg@goof.com>
+Marc Lehmann <pcg@goof.com>. The ColorSelectButton code is written by
+Dov Grobgeld <dov@imagic.weizmann.ac.il>, with modifications by Kenneth
+Albanowski <kjahds@kjahds.com>.
 
 =head1 SEE ALSO
 
